@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qwaylandinputdevice_p.h"
 
@@ -59,6 +23,7 @@
 #include "qwaylandcursor_p.h"
 #include "qwaylanddisplay_p.h"
 #include "qwaylandshmbackingstore_p.h"
+#include "qwaylandtextinputv1_p.h"
 #include "qwaylandtextinputv2_p.h"
 #if QT_WAYLAND_TEXT_INPUT_V4_WIP
 #include "qwaylandtextinputv4_p.h"
@@ -71,6 +36,7 @@
 #include <QtGui/private/qguiapplication_p.h>
 #include <qpa/qplatformwindow.h>
 #include <qpa/qplatforminputcontext.h>
+#include <qpa/qplatformtheme.h>
 #include <QDebug>
 
 #include <unistd.h>
@@ -264,19 +230,6 @@ private:
     QPoint m_hotspot;
 };
 
-QString QWaylandInputDevice::Pointer::cursorThemeName() const
-{
-    static QString themeName = qEnvironmentVariable("XCURSOR_THEME", QStringLiteral("default"));
-    return themeName;
-}
-
-int QWaylandInputDevice::Pointer::cursorSize() const
-{
-    constexpr int defaultCursorSize = 32;
-    static const int xCursorSize = qEnvironmentVariableIntValue("XCURSOR_SIZE");
-    return xCursorSize > 0 ? xCursorSize : defaultCursorSize;
-}
-
 int QWaylandInputDevice::Pointer::idealCursorScale() const
 {
     if (seat()->mQDisplay->compositor()->version() < 3) {
@@ -293,17 +246,30 @@ int QWaylandInputDevice::Pointer::idealCursorScale() const
 
 void QWaylandInputDevice::Pointer::updateCursorTheme()
 {
+    QString cursorThemeName;
+    QSize cursorSize;
+
+    if (const QPlatformTheme *platformTheme = QGuiApplicationPrivate::platformTheme()) {
+        cursorThemeName = platformTheme->themeHint(QPlatformTheme::MouseCursorTheme).toString();
+        cursorSize = platformTheme->themeHint(QPlatformTheme::MouseCursorSize).toSize();
+    }
+
+    if (cursorThemeName.isEmpty())
+        cursorThemeName = QStringLiteral("default");
+    if (cursorSize.isEmpty())
+        cursorSize = QSize(24, 24);
+
     int scale = idealCursorScale();
-    int pixelSize = cursorSize() * scale;
+    int pixelSize = cursorSize.width() * scale;
     auto *display = seat()->mQDisplay;
-    mCursor.theme = display->loadCursorTheme(cursorThemeName(), pixelSize);
+    mCursor.theme = display->loadCursorTheme(cursorThemeName, pixelSize);
 
     if (!mCursor.theme)
         return; // A warning has already been printed in loadCursorTheme
 
     if (auto *arrow = mCursor.theme->cursor(Qt::ArrowCursor)) {
         int arrowPixelSize = qMax(arrow->images[0]->width, arrow->images[0]->height); // Not all cursor themes are square
-        while (scale > 1 && arrowPixelSize / scale < cursorSize())
+        while (scale > 1 && arrowPixelSize / scale < cursorSize.width())
             --scale;
     } else {
         qCWarning(lcQpaWayland) << "Cursor theme does not support the arrow cursor";
@@ -319,8 +285,7 @@ void QWaylandInputDevice::Pointer::updateCursor()
     auto shape = seat()->mCursor.shape;
 
     if (shape == Qt::BlankCursor) {
-        if (mCursor.surface)
-            mCursor.surface->hide();
+        getOrCreateCursorSurface()->hide();
         return;
     }
 
@@ -410,7 +375,7 @@ QWaylandInputDevice::Touch::~Touch()
 }
 
 QWaylandInputDevice::QWaylandInputDevice(QWaylandDisplay *display, int version, uint32_t id)
-    : QtWayland::wl_seat(display->wl_registry(), id, qMin(version, 7))
+    : QtWayland::wl_seat(display->wl_registry(), id, qMin(version, 8))
     , mQDisplay(display)
     , mDisplay(display->wl_display())
 {
@@ -426,6 +391,12 @@ QWaylandInputDevice::QWaylandInputDevice(QWaylandDisplay *display, int version,
         setPrimarySelectionDevice(psm->createDevice(this));
 #endif
 
+    if (mQDisplay->textInputManagerv1()) {
+        auto textInput = new QWaylandTextInputv1(mQDisplay, mQDisplay->textInputManagerv1()->create_text_input());
+        textInput->setSeat(wl_seat());
+        mTextInput.reset(textInput);
+    }
+
     if (mQDisplay->textInputManagerv2())
         mTextInput.reset(new QWaylandTextInputv2(mQDisplay, mQDisplay->textInputManagerv2()->get_text_input(wl_seat())));
 
@@ -710,7 +681,7 @@ void QWaylandInputDevice::Pointer::pointer_enter(uint32_t serial, struct wl_surf
     connect(mFocus.data(), &QObject::destroyed, this, &Pointer::handleFocusDestroyed);
 
     mSurfacePos = QPointF(wl_fixed_to_double(sx), wl_fixed_to_double(sy));
-    mGlobalPos = window->window()->mapToGlobal(mSurfacePos.toPoint());
+    mGlobalPos = window->mapToGlobal(mSurfacePos.toPoint());
 
     mParent->mSerial = serial;
     mEnterSerial = serial;
@@ -776,7 +747,7 @@ void QWaylandInputDevice::Pointer::pointer_motion(uint32_t time, wl_fixed_t surf
 
     QPointF pos(wl_fixed_to_double(surface_x), wl_fixed_to_double(surface_y));
     QPointF delta = pos - pos.toPoint();
-    QPointF global = window->window()->mapToGlobal(pos.toPoint());
+    QPointF global = window->mapToGlobal(pos.toPoint());
     global += delta;
 
     mSurfacePos = pos;
@@ -788,7 +759,7 @@ void QWaylandInputDevice::Pointer::pointer_motion(uint32_t time, wl_fixed_t surf
         // We can't know the true position since we're getting events for another surface,
         // so we just set it outside of the window boundaries.
         pos = QPointF(-1, -1);
-        global = grab->window()->mapToGlobal(pos.toPoint());
+        global = grab->mapToGlobal(pos.toPoint());
         window = grab;
     }
     setFrameEvent(new MotionEvent(window, time, pos, global, mButtons, mParent->modifiers()));
@@ -868,7 +839,7 @@ void QWaylandInputDevice::Pointer::pointer_button(uint32_t serial, uint32_t time
     QPointF global = mGlobalPos;
     if (grab && grab != focusWindow()) {
         pos = QPointF(-1, -1);
-        global = grab->window()->mapToGlobal(pos.toPoint());
+        global = grab->mapToGlobal(pos.toPoint());
 
         window = grab;
     }
@@ -893,7 +864,7 @@ void QWaylandInputDevice::Pointer::releaseButtons()
     mButtons = Qt::NoButton;
 
     if (auto *window = focusWindow()) {
-        MotionEvent e(focusWindow(), mParent->mTime, mSurfacePos, mGlobalPos, mButtons, mParent->modifiers());
+        ReleaseEvent e(focusWindow(), mParent->mTime, mSurfacePos, mGlobalPos, mButtons, Qt::NoButton, mParent->modifiers());
         window->handleMouse(mParent, e);
     }
 }
@@ -1011,14 +982,16 @@ void QWaylandInputDevice::Pointer::pointer_axis_discrete(uint32_t axis, int32_t
     if (!focusWindow())
         return;
 
+    const int32_t delta120 = value * 15 * 8;
+
     switch (axis) {
     case axis_vertical_scroll:
         qCDebug(lcQpaWaylandInput) << "wl_pointer.axis_discrete vertical:" << value;
-        mFrameData.discreteDelta.ry() += value;
+        mFrameData.delta120.ry() += delta120;
         break;
     case axis_horizontal_scroll:
         qCDebug(lcQpaWaylandInput) << "wl_pointer.axis_discrete horizontal:" << value;
-        mFrameData.discreteDelta.rx() += value;
+        mFrameData.delta120.rx() += delta120;
         break;
     default:
         //TODO: is this really needed?
@@ -1027,6 +1000,26 @@ void QWaylandInputDevice::Pointer::pointer_axis_discrete(uint32_t axis, int32_t
     }
 }
 
+void QWaylandInputDevice::Pointer::pointer_axis_value120(uint32_t axis, int32_t value)
+{
+    if (!focusWindow())
+        return;
+
+    switch (axis) {
+    case axis_vertical_scroll:
+        qCDebug(lcQpaWaylandInput) << "wl_pointer.axis_value120 vertical:" << value;
+        mFrameData.delta120.ry() += value;
+        break;
+    case axis_horizontal_scroll:
+        qCDebug(lcQpaWaylandInput) << "wl_pointer.axis_value120 horizontal:" << value;
+        mFrameData.delta120.rx() += value;
+        break;
+    default:
+        qCWarning(lcQpaWaylandInput) << "wl_pointer.axis_value120: Unknown axis:" << axis;
+        return;
+    }
+}
+
 void QWaylandInputDevice::Pointer::setFrameEvent(QWaylandPointerEvent *event)
 {
     qCDebug(lcQpaWaylandInput) << "Setting frame event " << event->type;
@@ -1045,7 +1038,7 @@ void QWaylandInputDevice::Pointer::setFrameEvent(QWaylandPointerEvent *event)
 
 void QWaylandInputDevice::Pointer::FrameData::resetScrollData()
 {
-    discreteDelta = QPoint();
+    delta120 = QPoint();
     delta = QPointF();
     axisSource = axis_source_wheel;
 }
@@ -1089,15 +1082,16 @@ QPoint QWaylandInputDevice::Pointer::FrameData::pixelDeltaAndError(QPointF *accu
 
 QPoint QWaylandInputDevice::Pointer::FrameData::angleDelta() const
 {
-    if (discreteDelta.isNull()) {
+    if (delta120.isNull()) {
         // If we didn't get any discrete events, then we need to fall back to
         // the continuous information.
         return (delta * -12).toPoint(); //TODO: why multiply by 12?
     }
 
     // The angle delta is in eights of degrees, and our docs says most mice have
-    // 1 click = 15 degrees. It's also in the opposite direction of surface space.
-    return -discreteDelta * 15 * 8;
+    // 1 click = 15 degrees, i.e. 120 is one click. It's also in the opposite
+    // direction of surface space.
+    return -delta120;
 }
 
 Qt::MouseEventSource QWaylandInputDevice::Pointer::FrameData::wheelEventSource() const
@@ -1441,6 +1435,7 @@ void QWaylandInputDevice::Touch::touch_cancel()
     if (touchExt)
         touchExt->touchCanceled();
 
+    mFocus = nullptr;
     QWindowSystemInterface::handleTouchCancelEvent(nullptr, mParent->mTouchDevice);
 }
 
@@ -1480,7 +1475,7 @@ void QWaylandInputDevice::handleTouchPoint(int id, QEventPoint::State state, con
         // TODO: This doesn't account for high dpi scaling for the delta, but at least it matches
         // what we have for mouse input.
         QPointF delta = localPosition - localPosition.toPoint();
-        QPointF globalPosition = win->window()->mapToGlobal(localPosition.toPoint()) + delta;
+        QPointF globalPosition = win->mapToGlobal(localPosition.toPoint()) + delta;
         tp.area.moveCenter(globalPosition);
     }
 
@@ -1493,7 +1488,7 @@ void QWaylandInputDevice::handleTouchPoint(int id, QEventPoint::State state, con
 
 bool QWaylandInputDevice::Touch::allTouchPointsReleased()
 {
-    for (const auto &tp : qAsConst(mPendingTouchPoints)) {
+    for (const auto &tp : std::as_const(mPendingTouchPoints)) {
         if (tp.state != QEventPoint::Released)
             return false;
     }
@@ -1547,3 +1542,5 @@ void QWaylandInputDevice::Touch::touch_frame()
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qwaylandinputdevice_p.cpp"
